@@ -11,10 +11,11 @@
 *************************************************************************
 ** Code for testing the the SQLite library in a multithreaded environment.
 **
-** $Id: test4.c,v 1.3 2004/04/23 17:04:45 drh Exp $
+** $Id: test4.c,v 1.16 2006/01/20 17:56:33 drh Exp $
 */
 #include "sqliteInt.h"
 #include "tcl.h"
+#include "os.h"
 #if defined(OS_UNIX) && OS_UNIX==1 && defined(THREADSAFE) && THREADSAFE==1
 #include <stdlib.h>
 #include <string.h>
@@ -39,14 +40,14 @@ struct Thread {
   /* The next group of fields are writable by the thread but read-only to the
   ** master. */
   int completed;        /* Number of operations completed */
-  sqlite *db;           /* Open database */
-  sqlite_vm *vm;        /* Pending operation */
+  sqlite3 *db;           /* Open database */
+  sqlite3_stmt *pStmt;     /* Pending operation */
   char *zErr;           /* operation error */
   char *zStaticErr;     /* Static error message */
   int rc;               /* operation return code */
   int argc;             /* number of columns in result */
-  const char **argv;    /* result columns */
-  const char **colv;    /* result column names */
+  const char *argv[100];    /* result columns */
+  const char *colv[100];    /* result column names */
 };
 
 /*
@@ -63,34 +64,40 @@ static Thread threadset[N_THREAD];
 static void *thread_main(void *pArg){
   Thread *p = (Thread*)pArg;
   if( p->db ){
-    sqlite_close(p->db);
+    sqlite3_close(p->db);
   }
-  p->db = sqlite_open(p->zFilename, 0, &p->zErr);
-  p->vm = 0;
+  sqlite3_open(p->zFilename, &p->db);
+  if( SQLITE_OK!=sqlite3_errcode(p->db) ){
+    p->zErr = strdup(sqlite3_errmsg(p->db));
+    sqlite3_close(p->db);
+    p->db = 0;
+  }
+  p->pStmt = 0;
   p->completed = 1;
   while( p->opnum<=p->completed ) sched_yield();
   while( p->xOp ){
     if( p->zErr && p->zErr!=p->zStaticErr ){
-      sqlite_freemem(p->zErr);
+      sqlite3_free(p->zErr);
       p->zErr = 0;
     }
     (*p->xOp)(p);
     p->completed++;
     while( p->opnum<=p->completed ) sched_yield();
   }
-  if( p->vm ){
-    sqlite_finalize(p->vm, 0);
-    p->vm = 0;
+  if( p->pStmt ){
+    sqlite3_finalize(p->pStmt);
+    p->pStmt = 0;
   }
   if( p->db ){
-    sqlite_close(p->db);
+    sqlite3_close(p->db);
     p->db = 0;
   }
   if( p->zErr && p->zErr!=p->zStaticErr ){
-    sqlite_freemem(p->zErr);
+    sqlite3_free(p->zErr);
     p->zErr = 0;
   }
   p->completed++;
+  sqlite3_thread_cleanup();
   return 0;
 }
 
@@ -100,7 +107,7 @@ static void *thread_main(void *pArg){
 ** the interpreter and return -1.
 */
 static int parse_thread_id(Tcl_Interp *interp, const char *zArg){
-  if( zArg==0 || zArg[0]==0 || zArg[1]!=0 || !isupper(zArg[0]) ){
+  if( zArg==0 || zArg[0]==0 || zArg[1]!=0 || !isupper((unsigned char)zArg[0]) ){
     Tcl_AppendResult(interp, "thread ID must be an upper case letter", 0);
     return -1;
   }
@@ -369,7 +376,6 @@ static int tcl_thread_result(
   switch( threadset[i].rc ){
     case SQLITE_OK:         zName = "SQLITE_OK";          break;
     case SQLITE_ERROR:      zName = "SQLITE_ERROR";       break;
-    case SQLITE_INTERNAL:   zName = "SQLITE_INTERNAL";    break;
     case SQLITE_PERM:       zName = "SQLITE_PERM";        break;
     case SQLITE_ABORT:      zName = "SQLITE_ABORT";       break;
     case SQLITE_BUSY:       zName = "SQLITE_BUSY";        break;
@@ -379,13 +385,11 @@ static int tcl_thread_result(
     case SQLITE_INTERRUPT:  zName = "SQLITE_INTERRUPT";   break;
     case SQLITE_IOERR:      zName = "SQLITE_IOERR";       break;
     case SQLITE_CORRUPT:    zName = "SQLITE_CORRUPT";     break;
-    case SQLITE_NOTFOUND:   zName = "SQLITE_NOTFOUND";    break;
     case SQLITE_FULL:       zName = "SQLITE_FULL";        break;
     case SQLITE_CANTOPEN:   zName = "SQLITE_CANTOPEN";    break;
     case SQLITE_PROTOCOL:   zName = "SQLITE_PROTOCOL";    break;
     case SQLITE_EMPTY:      zName = "SQLITE_EMPTY";       break;
     case SQLITE_SCHEMA:     zName = "SQLITE_SCHEMA";      break;
-    case SQLITE_TOOBIG:     zName = "SQLITE_TOOBIG";      break;
     case SQLITE_CONSTRAINT: zName = "SQLITE_CONSTRAINT";  break;
     case SQLITE_MISMATCH:   zName = "SQLITE_MISMATCH";    break;
     case SQLITE_MISUSE:     zName = "SQLITE_MISUSE";      break;
@@ -440,11 +444,11 @@ static void do_compile(Thread *p){
     p->rc = SQLITE_ERROR;
     return;
   }
-  if( p->vm ){
-    sqlite_finalize(p->vm, 0);
-    p->vm = 0;
+  if( p->pStmt ){
+    sqlite3_finalize(p->pStmt);
+    p->pStmt = 0;
   }
-  p->rc = sqlite_compile(p->db, p->zArg, 0, &p->vm, &p->zErr);
+  p->rc = sqlite3_prepare(p->db, p->zArg, -1, &p->pStmt, 0);
 }
 
 /*
@@ -482,12 +486,22 @@ static int tcl_thread_compile(
 ** This procedure runs in the thread to step the virtual machine.
 */
 static void do_step(Thread *p){
-  if( p->vm==0 ){
+  int i;
+  if( p->pStmt==0 ){
     p->zErr = p->zStaticErr = "no virtual machine available";
     p->rc = SQLITE_ERROR;
     return;
   }
-  p->rc = sqlite_step(p->vm, &p->argc, &p->argv, &p->colv);
+  p->rc = sqlite3_step(p->pStmt);
+  if( p->rc==SQLITE_ROW ){
+    p->argc = sqlite3_column_count(p->pStmt);
+    for(i=0; i<sqlite3_data_count(p->pStmt); i++){
+      p->argv[i] = (char*)sqlite3_column_text(p->pStmt, i);
+    }
+    for(i=0; i<p->argc; i++){
+      p->colv[i] = sqlite3_column_name(p->pStmt, i);
+    }
+  }
 }
 
 /*
@@ -523,13 +537,13 @@ static int tcl_thread_step(
 ** This procedure runs in the thread to finalize a virtual machine.
 */
 static void do_finalize(Thread *p){
-  if( p->vm==0 ){
+  if( p->pStmt==0 ){
     p->zErr = p->zStaticErr = "no virtual machine available";
     p->rc = SQLITE_ERROR;
     return;
   }
-  p->rc = sqlite_finalize(p->vm, &p->zErr);
-  p->vm = 0;
+  p->rc = sqlite3_finalize(p->pStmt);
+  p->pStmt = 0;
 }
 
 /*
@@ -575,7 +589,7 @@ static int tcl_thread_swap(
   const char **argv      /* Text of each argument */
 ){
   int i, j;
-  sqlite *temp;
+  sqlite3 *temp;
   if( argc!=3 ){
     Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
        " ID1 ID2", 0);
@@ -601,6 +615,73 @@ static int tcl_thread_swap(
   return TCL_OK;
 }
 
+/*
+** Usage: thread_db_get ID
+**
+** Return the database connection pointer for the given thread.  Then
+** remove the pointer from the thread itself.  Afterwards, the thread
+** can be stopped and the connection can be used by the main thread.
+*/
+static int tcl_thread_db_get(
+  void *NotUsed,
+  Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
+  int argc,              /* Number of arguments */
+  const char **argv      /* Text of each argument */
+){
+  int i;
+  char zBuf[100];
+  extern int sqlite3TestMakePointerStr(Tcl_Interp*, char*, void*);
+  if( argc!=2 ){
+    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
+       " ID", 0);
+    return TCL_ERROR;
+  }
+  i = parse_thread_id(interp, argv[1]);
+  if( i<0 ) return TCL_ERROR;
+  if( !threadset[i].busy ){
+    Tcl_AppendResult(interp, "no such thread", 0);
+    return TCL_ERROR;
+  }
+  thread_wait(&threadset[i]);
+  sqlite3TestMakePointerStr(interp, zBuf, threadset[i].db);
+  threadset[i].db = 0;
+  Tcl_SetResult(interp, zBuf, 0);
+  return TCL_OK;
+}
+
+/*
+** Usage: thread_stmt_get ID
+**
+** Return the database stmt pointer for the given thread.  Then
+** remove the pointer from the thread itself. 
+*/
+static int tcl_thread_stmt_get(
+  void *NotUsed,
+  Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
+  int argc,              /* Number of arguments */
+  const char **argv      /* Text of each argument */
+){
+  int i;
+  char zBuf[100];
+  extern int sqlite3TestMakePointerStr(Tcl_Interp*, char*, void*);
+  if( argc!=2 ){
+    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
+       " ID", 0);
+    return TCL_ERROR;
+  }
+  i = parse_thread_id(interp, argv[1]);
+  if( i<0 ) return TCL_ERROR;
+  if( !threadset[i].busy ){
+    Tcl_AppendResult(interp, "no such thread", 0);
+    return TCL_ERROR;
+  }
+  thread_wait(&threadset[i]);
+  sqlite3TestMakePointerStr(interp, zBuf, threadset[i].pStmt);
+  threadset[i].pStmt = 0;
+  Tcl_SetResult(interp, zBuf, 0);
+  return TCL_OK;
+}
+
 /*
 ** Register commands with the TCL interpreter.
 */
@@ -621,6 +702,8 @@ int Sqlitetest4_Init(Tcl_Interp *interp){
      { "thread_step",       (Tcl_CmdProc*)tcl_thread_step       },
      { "thread_finalize",   (Tcl_CmdProc*)tcl_thread_finalize   },
      { "thread_swap",       (Tcl_CmdProc*)tcl_thread_swap       },
+     { "thread_db_get",     (Tcl_CmdProc*)tcl_thread_db_get     },
+     { "thread_stmt_get",   (Tcl_CmdProc*)tcl_thread_stmt_get   },
   };
   int i;
 
