@@ -105,17 +105,13 @@ typedef struct AsyncWrite AsyncWrite;
 typedef struct AsyncFile AsyncFile;
 
 /* Enable for debugging */
-static int sqlite3async_trace = 0;
-# define TRACE(X) if( sqlite3async_trace ) asyncTrace X
-static void asyncTrace(const char *zFormat, ...){
-  char *z;
-  va_list ap;
-  va_start(ap, zFormat);
-  z = sqlite3_vmprintf(zFormat, ap);
-  va_end(ap);
-  fprintf(stderr, "[%d] %s", (int)pthread_self(), z);
-  free(z);
-}
+#if 0
+# define TRACE(X,Y) \
+    fprintf(stderr,"THRD=%d: ", (int)pthread_self()); \
+    fprintf(stderr,X,Y);
+#else
+# define TRACE(X,Y) /* noop */
+#endif
 
 /*
 ** THREAD SAFETY NOTES
@@ -161,10 +157,8 @@ static void asyncTrace(const char *zFormat, ...){
 **
 **         asyncLock, asyncUnlock, asyncLockState, asyncCheckReservedLock
 **    
-**     These primitives implement in-process locking using a hash table
-**     on the file name.  Files are locked correctly for connections coming
-**     from the same process.  But other processes cannot see these locks
-**     and will therefore not honor them.
+**     These locking primitives become no-ops. Files are always opened for 
+**     exclusive access when using this IO backend.  
 **
 **
 **         asyncFileHandle.
@@ -235,19 +229,14 @@ static void asyncTrace(const char *zFormat, ...){
 static struct TestAsyncStaticData {
   pthread_mutex_t queueMutex;  /* Mutex for access to write operation queue */
   pthread_mutex_t writerMutex; /* Prevents multiple writer threads */
-  pthread_mutex_t lockMutex;   /* For access to aLock hash table */
   pthread_cond_t queueSignal;  /* For waking up sleeping writer thread */
   pthread_cond_t emptySignal;  /* Notify when the write queue is empty */
   AsyncWrite *pQueueFirst;     /* Next write operation to be processed */
   AsyncWrite *pQueueLast;      /* Last write operation on the list */
-  Hash aLock;                  /* Files locked */
   volatile int ioDelay;             /* Extra delay between write operations */
   volatile int writerHaltWhenIdle;  /* Writer thread halts when queue empty */
   volatile int writerHaltNow;       /* Writer thread halts after next op */
-  int ioError;                 /* True if an IO error has occured */
-  int nFile;                   /* Number of open files (from sqlite pov) */
 } async = {
-  PTHREAD_MUTEX_INITIALIZER,
   PTHREAD_MUTEX_INITIALIZER,
   PTHREAD_MUTEX_INITIALIZER,
   PTHREAD_COND_INITIALIZER,
@@ -255,25 +244,17 @@ static struct TestAsyncStaticData {
 };
 
 /* Possible values of AsyncWrite.op */
-#define ASYNC_NOOP          0
 #define ASYNC_WRITE         1
 #define ASYNC_SYNC          2
 #define ASYNC_TRUNCATE      3
 #define ASYNC_CLOSE         4
 #define ASYNC_OPENDIRECTORY 5
 #define ASYNC_SETFULLSYNC   6
+
 #define ASYNC_DELETE        7
 #define ASYNC_OPENEXCLUSIVE 8
 #define ASYNC_SYNCDIRECTORY 9
 
-/* Names of opcodes.  Used for debugging only.
-** Make sure these stay in sync with the macros above!
-*/
-static const char *azOpcodeName[] = {
-  "NOOP", "WRITE", "SYNC", "TRUNCATE", "CLOSE",
-  "OPENDIR", "SETFULLSYNC", "DELETE", "OPENEX", "SYNCDIR",
-};
-
 /*
 ** Entries on the write-op queue are instances of the AsyncWrite
 ** structure, defined here.
@@ -314,7 +295,6 @@ static const char *azOpcodeName[] = {
 **     iOffset -> Value of "delflag".
 **     nByte   -> Number of bytes of zBuf points to (file name).
 **
-**
 ** For an ASYNC_WRITE operation, zBuf points to the data to write to the file. 
 ** This space is sqliteMalloc()d along with the AsyncWrite structure in a
 ** single blob, so is deleted when sqliteFree() is called on the parent 
@@ -335,8 +315,6 @@ struct AsyncWrite {
 struct AsyncFile {
   IoMethod *pMethod;   /* Must be first */
   i64 iOffset;         /* Current seek() offset in file */
-  char *zName;         /* Underlying OS filename - used for debugging */
-  int nName;           /* Number of characters in zName */
   OsFile *pBaseRead;   /* Read handle to the underlying Os file */
   OsFile *pBaseWrite;  /* Write handle to the underlying Os file */
 };
@@ -364,15 +342,7 @@ static void addAsyncWrite(AsyncWrite *pWrite){
     async.pQueueFirst = pWrite;
   }
   async.pQueueLast = pWrite;
-  TRACE(("PUSH %p (%s %s %d)\n", pWrite, azOpcodeName[pWrite->op],
-         pWrite->pFile ? pWrite->pFile->zName : "-", pWrite->iOffset));
-
-  if( pWrite->op==ASYNC_CLOSE ){
-    async.nFile--;
-    if( async.nFile==0 ){
-      async.ioError = SQLITE_OK;
-    }
-  }
+  TRACE("PUSH %p\n", pWrite);
 
   /* Drop the queue mutex */
   pthread_mutex_unlock(&async.queueMutex);
@@ -382,19 +352,6 @@ static void addAsyncWrite(AsyncWrite *pWrite){
   pthread_cond_signal(&async.queueSignal);
 }
 
-/*
-** Increment async.nFile in a thread-safe manner.
-*/
-static void incrOpenFileCount(){
-  /* We must hold the queue mutex in order to modify async.nFile */
-  pthread_mutex_lock(&async.queueMutex);
-  if( async.nFile==0 ){
-    async.ioError = SQLITE_OK;
-  }
-  async.nFile++;
-  pthread_mutex_unlock(&async.queueMutex);
-}
-
 /*
 ** This is a utility function to allocate and populate a new AsyncWrite
 ** structure and insert it (via addAsyncWrite() ) into the global list.
@@ -406,11 +363,7 @@ static int addNewAsyncWrite(
   int nByte,
   const char *zByte
 ){
-  AsyncWrite *p;
-  if( op!=ASYNC_CLOSE && async.ioError ){
-    return async.ioError;
-  }
-  p = sqlite3OsMalloc(sizeof(AsyncWrite) + (zByte?nByte:0));
+  AsyncWrite *p = sqlite3OsMalloc(sizeof(AsyncWrite) + (zByte?nByte:0));
   if( !p ){
     return SQLITE_NOMEM;
   }
@@ -496,31 +449,22 @@ static int asyncRead(OsFile *id, void *obuf, int amt){
   i64 filesize;
   int nRead;
   AsyncFile *pFile = (AsyncFile *)id;
-  OsFile *pBase = pFile->pBaseRead;
-
-  /* If an I/O error has previously occurred on this file, then all
-  ** subsequent operations fail.
-  */
-  if( async.ioError!=SQLITE_OK ){
-    return async.ioError;
-  }
 
   /* Grab the write queue mutex for the duration of the call */
   pthread_mutex_lock(&async.queueMutex);
 
-  if( pBase ){
-    rc = sqlite3OsFileSize(pBase, &filesize);
+  if( pFile->pBaseRead ){
+    rc = sqlite3OsFileSize(pFile->pBaseRead, &filesize);
     if( rc!=SQLITE_OK ){
       goto asyncread_out;
     }
-    rc = sqlite3OsSeek(pBase, pFile->iOffset);
+    rc = sqlite3OsSeek(pFile->pBaseRead, pFile->iOffset);
     if( rc!=SQLITE_OK ){
       goto asyncread_out;
     }
     nRead = MIN(filesize - pFile->iOffset, amt);
     if( nRead>0 ){
-      rc = sqlite3OsRead(pBase, obuf, nRead);
-      TRACE(("READ %s %d bytes at %d\n", pFile->zName, nRead, pFile->iOffset));
+      rc = sqlite3OsRead(((AsyncFile *)id)->pBaseRead, obuf, nRead);
     }
   }
 
@@ -530,8 +474,8 @@ static int asyncRead(OsFile *id, void *obuf, int amt){
 
     for(p=async.pQueueFirst; p; p = p->pNext){
       if( p->pFile==pFile && p->op==ASYNC_WRITE ){
-        int iBeginOut = (p->iOffset - iOffset);
-        int iBeginIn = -iBeginOut;
+        int iBeginIn = (p->iOffset - iOffset);
+        int iBeginOut = (iOffset - p->iOffset);
         int nCopy;
 
         if( iBeginIn<0 ) iBeginIn = 0;
@@ -540,7 +484,6 @@ static int asyncRead(OsFile *id, void *obuf, int amt){
 
         if( nCopy>0 ){
           memcpy(&((char *)obuf)[iBeginOut], &p->zBuf[iBeginIn], nCopy);
-          TRACE(("OVERREAD %d bytes at %d\n", nCopy, iBeginOut+iOffset));
         }
       }
     }
@@ -597,7 +540,7 @@ int asyncFileSize(OsFile *id, i64 *pSize){
             s = MAX(p->iOffset + (i64)(p->nByte), s);
             break;
           case ASYNC_TRUNCATE:
-            s = MIN(s, p->iOffset);
+            s = MIN(s, p->nByte);
             break;
         }
       }
@@ -617,23 +560,14 @@ static int asyncFileHandle(OsFile *id){
 }
 
 /*
-** No disk locking is performed.  We keep track of locks locally in
-** the async.aLock hash table.  Locking should appear to work the same
-** as with standard (unmodified) SQLite as long as all connections 
-** come from this one process.  Connections from external processes
-** cannot see our internal hash table (obviously) and will thus not
-** honor our locks.
+** No file locking occurs with this version of the asynchronous backend.
+** So the locking routines are no-ops.
 */
 static int asyncLock(OsFile *id, int lockType){
-  AsyncFile *pFile = (AsyncFile*)id;
-  TRACE(("LOCK %d (%s)\n", lockType, pFile->zName));
-  pthread_mutex_lock(&async.lockMutex);
-  sqlite3HashInsert(&async.aLock, pFile->zName, pFile->nName, (void*)lockType);
-  pthread_mutex_unlock(&async.lockMutex);
   return SQLITE_OK;
 }
 static int asyncUnlock(OsFile *id, int lockType){
-  return asyncLock(id, lockType);
+  return SQLITE_OK;
 }
 
 /*
@@ -641,13 +575,7 @@ static int asyncUnlock(OsFile *id, int lockType){
 ** and is checking for a hot-journal.
 */
 static int asyncCheckReservedLock(OsFile *id){
-  AsyncFile *pFile = (AsyncFile*)id;
-  int rc;
-  pthread_mutex_lock(&async.lockMutex);
-  rc = (int)sqlite3HashFind(&async.aLock, pFile->zName, pFile->nName);
-  pthread_mutex_unlock(&async.lockMutex);
-  TRACE(("CHECK-LOCK %d (%s)\n", rc, pFile->zName));
-  return rc>SHARED_LOCK;
+  return SQLITE_OK;
 }
 
 /* 
@@ -679,7 +607,7 @@ static int asyncOpenFile(
   OsFile *pBaseRead,     /* The real OsFile from the real I/O routine */
   int openForWriting     /* Open a second file handle for writing if true */
 ){
-  int rc, i, n;
+  int rc;
   AsyncFile *p;
   OsFile *pBaseWrite = 0;
 
@@ -708,17 +636,13 @@ static int asyncOpenFile(
     }
   }
 
-  n = strlen(zName);
-  for(i=n-1; i>=0 && zName[i]!='/'; i--){}
-  p = (AsyncFile *)sqlite3OsMalloc(sizeof(AsyncFile) + n - i);
+  p = (AsyncFile *)sqlite3OsMalloc(sizeof(AsyncFile));
   if( !p ){
     rc = SQLITE_NOMEM;
     goto error_out;
   }
   memset(p, 0, sizeof(AsyncFile));
-  p->zName = (char*)&p[1];
-  strcpy(p->zName, &zName[i+1]);
-  p->nName = n - i;
+  
   p->pMethod = &iomethod;
   p->pBaseRead = pBaseRead;
   p->pBaseWrite = pBaseWrite;
@@ -751,9 +675,6 @@ static int asyncOpenExclusive(const char *z, OsFile **ppFile, int delFlag){
       *ppFile = 0;
     }
   }
-  if( rc==SQLITE_OK ){
-    incrOpenFileCount();
-  }
   return rc;
 }
 static int asyncOpenReadOnly(const char *z, OsFile **ppFile){
@@ -762,9 +683,6 @@ static int asyncOpenReadOnly(const char *z, OsFile **ppFile){
   if( rc==SQLITE_OK ){
     rc = asyncOpenFile(z, ppFile, pBase, 0);
   }
-  if( rc==SQLITE_OK ){
-    incrOpenFileCount();
-  }
   return rc;
 }
 static int asyncOpenReadWrite(const char *z, OsFile **ppFile, int *pReadOnly){
@@ -773,9 +691,6 @@ static int asyncOpenReadWrite(const char *z, OsFile **ppFile, int *pReadOnly){
   if( rc==SQLITE_OK ){
     rc = asyncOpenFile(z, ppFile, pBase, (*pReadOnly ? 0 : 1));
   }
-  if( rc==SQLITE_OK ){
-    incrOpenFileCount();
-  }
   return rc;
 }
 
@@ -818,7 +733,6 @@ static int asyncFileExists(const char *z){
     }
   }
 
-  TRACE(("EXISTS: %s = %d\n", z, ret));
   pthread_mutex_unlock(&async.queueMutex);
   return ret;
 }
@@ -832,8 +746,6 @@ static int asyncFileExists(const char *z){
 */
 static void asyncEnable(int enable){
   if( enable && xOrigOpenReadWrite==0 ){
-    assert(sqlite3Os.xOpenReadWrite);
-    sqlite3HashInit(&async.aLock, SQLITE_HASH_BINARY, 1);
     xOrigOpenReadWrite = sqlite3Os.xOpenReadWrite;
     xOrigOpenReadOnly = sqlite3Os.xOpenReadOnly;
     xOrigOpenExclusive = sqlite3Os.xOpenExclusive;
@@ -847,11 +759,8 @@ static void asyncEnable(int enable){
     sqlite3Os.xDelete = asyncDelete;
     sqlite3Os.xFileExists = asyncFileExists;
     sqlite3Os.xSyncDirectory = asyncSyncDirectory;
-    assert(sqlite3Os.xOpenReadWrite);
   }
   if( !enable && xOrigOpenReadWrite!=0 ){
-    assert(sqlite3Os.xOpenReadWrite);
-    sqlite3HashClear(&async.aLock);
     sqlite3Os.xOpenReadWrite = xOrigOpenReadWrite;
     sqlite3Os.xOpenReadOnly = xOrigOpenReadOnly;
     sqlite3Os.xOpenExclusive = xOrigOpenExclusive;
@@ -865,7 +774,6 @@ static void asyncEnable(int enable){
     xOrigDelete = 0;
     xOrigFileExists = 0;
     xOrigSyncDirectory = 0;
-    assert(sqlite3Os.xOpenReadWrite);
   }
 }
 
@@ -891,30 +799,29 @@ static void asyncEnable(int enable){
 static void *asyncWriterThread(void *NotUsed){
   AsyncWrite *p = 0;
   int rc = SQLITE_OK;
-  int holdingMutex = 0;
 
   if( pthread_mutex_trylock(&async.writerMutex) ){
     return 0;
   }
   while( async.writerHaltNow==0 ){
+    int holdingMutex;
     OsFile *pBase = 0;
 
-    if( !holdingMutex ){
-      pthread_mutex_lock(&async.queueMutex);
-    }
+    pthread_mutex_lock(&async.queueMutex);
+    holdingMutex = 1;
     while( (p = async.pQueueFirst)==0 ){
       pthread_cond_broadcast(&async.emptySignal);
       if( async.writerHaltWhenIdle ){
         pthread_mutex_unlock(&async.queueMutex);
         break;
       }else{
-        TRACE(("IDLE\n"));
+        TRACE("IDLE\n", 0);
         pthread_cond_wait(&async.queueSignal, &async.queueMutex);
-        TRACE(("WAKEUP\n"));
+        TRACE("WAKEUP\n", 0);
       }
     }
     if( p==0 ) break;
-    holdingMutex = 1;
+    TRACE("PROCESSING %p\n", p);
 
     /* Right now this thread is holding the mutex on the write-op queue.
     ** Variable 'p' points to the first entry in the write-op queue. In
@@ -935,9 +842,6 @@ static void *asyncWriterThread(void *NotUsed){
     **       SQLITE_ASYNC_TWO_FILEHANDLES was set at compile time and two
     **       file-handles are open for the particular file being "synced".
     */
-    if( async.ioError!=SQLITE_OK && p->op!=ASYNC_CLOSE ){
-      p->op = ASYNC_NOOP;
-    }
     if( p->pFile ){
       pBase = p->pFile->pBaseWrite;
       if( 
@@ -954,13 +858,8 @@ static void *asyncWriterThread(void *NotUsed){
     }
 
     switch( p->op ){
-      case ASYNC_NOOP:
-        break;
-
       case ASYNC_WRITE:
         assert( pBase );
-        TRACE(("WRITE %s %d bytes at %d\n",
-                p->pFile->zName, p->nByte, p->iOffset));
         rc = sqlite3OsSeek(pBase, p->iOffset);
         if( rc==SQLITE_OK ){
           rc = sqlite3OsWrite(pBase, (const void *)(p->zBuf), p->nByte);
@@ -969,42 +868,35 @@ static void *asyncWriterThread(void *NotUsed){
 
       case ASYNC_SYNC:
         assert( pBase );
-        TRACE(("SYNC %s\n", p->pFile->zName));
         rc = sqlite3OsSync(pBase, p->nByte);
         break;
 
       case ASYNC_TRUNCATE:
         assert( pBase );
-        TRACE(("TRUNCATE %s to %d bytes\n", p->pFile->zName, p->iOffset));
-        rc = sqlite3OsTruncate(pBase, p->iOffset);
+        rc = sqlite3OsTruncate(pBase, p->nByte);
         break;
 
       case ASYNC_CLOSE:
-        TRACE(("CLOSE %s\n", p->pFile->zName));
-        sqlite3OsClose(&p->pFile->pBaseWrite);
         sqlite3OsClose(&p->pFile->pBaseRead);
+        sqlite3OsClose(&p->pFile->pBaseWrite);
         sqlite3OsFree(p->pFile);
         break;
 
       case ASYNC_OPENDIRECTORY:
         assert( pBase );
-        TRACE(("OPENDIR %s\n", p->zBuf));
         sqlite3OsOpenDirectory(pBase, p->zBuf);
         break;
 
       case ASYNC_SETFULLSYNC:
         assert( pBase );
-        TRACE(("SETFULLSYNC %s %d\n", p->pFile->zName, p->nByte));
         sqlite3OsSetFullSync(pBase, p->nByte);
         break;
 
       case ASYNC_DELETE:
-        TRACE(("DELETE %s\n", p->zBuf));
         rc = xOrigDelete(p->zBuf);
         break;
 
       case ASYNC_SYNCDIRECTORY:
-        TRACE(("SYNCDIR %s\n", p->zBuf));
         rc = xOrigSyncDirectory(p->zBuf);
         break;
 
@@ -1012,8 +904,6 @@ static void *asyncWriterThread(void *NotUsed){
         AsyncFile *pFile = p->pFile;
         int delFlag = ((p->iOffset)?1:0);
         OsFile *pBase = 0;
-        TRACE(("OPEN %s delFlag=%d\n", p->zBuf, delFlag));
-        assert(pFile->pBaseRead==0 && pFile->pBaseWrite==0);
         rc = xOrigOpenExclusive(p->zBuf, &pBase, delFlag);
         assert( holdingMutex==0 );
         pthread_mutex_lock(&async.queueMutex);
@@ -1035,51 +925,25 @@ static void *asyncWriterThread(void *NotUsed){
       pthread_mutex_lock(&async.queueMutex);
       holdingMutex = 1;
     }
-    /* TRACE(("UNLINK %p\n", p)); */
-    if( p==async.pQueueLast ){
-      async.pQueueLast = 0;
+    TRACE("UNLINK %p\n", p);
+    if( rc==SQLITE_OK ){
+      if( p==async.pQueueLast ){
+        async.pQueueLast = 0;
+      }
+      async.pQueueFirst = p->pNext;
     }
-    async.pQueueFirst = p->pNext;
-    sqlite3OsFree(p);
     assert( holdingMutex );
 
-    /* An IO error has occured. We cannot report the error back to the
-    ** connection that requested the I/O since the error happened 
-    ** asynchronously.  The connection has already moved on.  There 
-    ** really is nobody to report the error to.
-    **
-    ** The file for which the error occured may have been a database or
-    ** journal file. Regardless, none of the currently queued operations
-    ** associated with the same database should now be performed. Nor should
-    ** any subsequently requested IO on either a database or journal file 
-    ** handle for the same database be accepted until the main database
-    ** file handle has been closed and reopened.
-    **
-    ** Furthermore, no further IO should be queued or performed on any file
-    ** handle associated with a database that may have been part of a 
-    ** multi-file transaction that included the database associated with 
-    ** the IO error (i.e. a database ATTACHed to the same handle at some 
-    ** point in time).
-    */
-    if( rc!=SQLITE_OK ){
-      async.ioError = rc;
-    }
-
     /* Drop the queue mutex before continuing to the next write operation
     ** in order to give other threads a chance to work with the write queue.
     */
-    if( !async.pQueueFirst || !async.ioError ){
-      sqlite3ApiExit(0, 0);
-      pthread_mutex_unlock(&async.queueMutex);
-      holdingMutex = 0;
-      if( async.ioDelay>0 ){
-        sqlite3OsSleep(async.ioDelay);
-      }else{
-        sched_yield();
-      }
+    pthread_mutex_unlock(&async.queueMutex);
+    if( async.ioDelay>0 ){
+      sqlite3OsSleep(async.ioDelay);
+    }else{
+      sched_yield();
     }
   }
-  
   pthread_mutex_unlock(&async.writerMutex);
   return 0;
 }
@@ -1217,26 +1081,14 @@ static int testAsyncWait(
   int objc,
   Tcl_Obj *CONST objv[]
 ){
-  int cnt = 10;
   if( async.writerHaltNow==0 && async.writerHaltWhenIdle==0 ){
     Tcl_AppendResult(interp, "would block forever", (char*)0);
     return TCL_ERROR;
   }
-
-  while( cnt-- && !pthread_mutex_trylock(&async.writerMutex) ){
-    pthread_mutex_unlock(&async.writerMutex);
-    sched_yield();
-  }
-  if( cnt>=0 ){
-    TRACE(("WAIT\n"));
-    pthread_mutex_lock(&async.queueMutex);
-    pthread_cond_broadcast(&async.queueSignal);
-    pthread_mutex_unlock(&async.queueMutex);
-    pthread_mutex_lock(&async.writerMutex);
-    pthread_mutex_unlock(&async.writerMutex);
-  }else{
-    TRACE(("NO-WAIT\n"));
-  }
+  TRACE("WAIT\n",0);
+  pthread_cond_broadcast(&async.queueSignal);
+  pthread_mutex_lock(&async.writerMutex);
+  pthread_mutex_unlock(&async.writerMutex);
   return TCL_OK;
 }
 
@@ -1255,8 +1107,6 @@ int Sqlitetestasync_Init(Tcl_Interp *interp){
   Tcl_CreateObjCommand(interp,"sqlite3async_delay",testAsyncDelay,0,0);
   Tcl_CreateObjCommand(interp,"sqlite3async_start",testAsyncStart,0,0);
   Tcl_CreateObjCommand(interp,"sqlite3async_wait",testAsyncWait,0,0);
-  Tcl_LinkVar(interp, "sqlite3async_trace",
-      (char*)&sqlite3async_trace, TCL_LINK_INT);
 #endif  /* OS_UNIX and THREADSAFE and defined(SQLITE_ENABLE_REDEF_IO) */
   return TCL_OK;
 }
