@@ -1,4 +1,4 @@
-set rcsid {$Id: capi3ref.tcl,v 1.38 2006/04/05 01:08:35 drh Exp $}
+set rcsid {$Id: capi3ref.tcl,v 1.34 2006/02/11 01:25:52 drh Exp $}
 source common.tcl
 header {C/C++ Interface For SQLite Version 3}
 puts {
@@ -176,7 +176,7 @@ api {} {
  If the busy callback is NULL, then SQLITE_BUSY is returned immediately
  upon encountering the lock.
  If the busy callback is not NULL, then the
- callback will be invoked with two arguments.  The
+ callback might invoked with two arguments.  The
  second argument is the number of prior calls to the busy callback
  for the same lock.  If the
  busy callback returns 0, then no additional attempts are made to
@@ -988,11 +988,6 @@ int sqlite3_open16(
  pass the resulting database handle off to another thread to use.  This
  restriction is due to goofy design decisions (bugs?) in the way some
  threading implementations interact with file locks.
-
- Note to windows users:  The encoding used for the filename argument
- of sqlite3_open() must be UTF-8, not whatever codepage is currently
- defined.  Filenames containing international characters must be converted
- to UTF-8 prior to passing them into sqlite3_open().
 }
 
 api {} {
@@ -1133,9 +1128,6 @@ int sqlite3_set_authorizer(
 #define SQLITE_UPDATE               23   /* Table Name      Column Name     */
 #define SQLITE_ATTACH               24   /* Filename        NULL            */
 #define SQLITE_DETACH               25   /* Database Name   NULL            */
-#define SQLITE_ALTER_TABLE          26   /* Database Name   Table Name      */
-#define SQLITE_REINDEX              27   /* Index Name      NULL            */
-#define SQLITE_ANALYZE              28   /* Table Name      NULL            */
 
 #define SQLITE_DENY   1   /* Abort the SQL statement with an error */
 #define SQLITE_IGNORE 2   /* Don't allow access, but don't generate an error */
@@ -1304,7 +1296,7 @@ api {} {
 }
 
 api {} {
-  sqlite3 *sqlite3_db_handle(sqlite3_stmt*);
+  int sqlite3_db_handle(sqlite3_stmt*);
 } {
  Return the sqlite3* database handle to which the prepared statement given
  in the argument belongs.  This is the same database handle that was
