@@ -1,4 +1,4 @@
-set rcsid {$Id: datatype3.tcl,v 1.13 2006/04/01 14:38:41 drh Exp $}
+set rcsid {$Id: datatype3.tcl,v 1.12 2006/01/26 13:25:08 danielk1977 Exp $}
 source common.tcl
 header {Datatypes In SQLite Version 3}
 puts {
@@ -312,12 +312,13 @@ instead of memcmp().
 </p>  
 <p>
 Aside from the default collation sequence BINARY, implemented using
-memcmp(), SQLite features one extra built-in collation sequences 
-intended for testing purposes, the NOCASE collation:
+memcmp(), SQLite features two extra built-in collation sequences 
+intended for testing purposes, NOCASE and REVERSE:
 </p>  
 <UL>
 	<LI><b>BINARY</b> - Compares string data using memcmp(), regardless
                             of text encoding.</LI>
+	<LI><b>REVERSE</b> - Collate in the reverse order to BINARY. </LI>
 	<LI><b>NOCASE</b> - The same as binary, except the 26 upper case
 			    characters used by the English language are
 			    folded to their lower case equivalents before
