@@ -1,7 +1,7 @@
 #
 # Run this Tcl script to generate the lang-*.html files.
 #
-set rcsid {$Id: lang.tcl,v 1.89 2005/03/21 01:24:02 drh Exp $}
+set rcsid {$Id: lang.tcl,v 1.108 2006/02/09 22:13:42 drh Exp $}
 source common.tcl
 
 if {[llength $argv]>0} {
@@ -76,6 +76,7 @@ foreach {section} [lsort -index 0 -dictionary {
   {{DETACH DATABASE} detach}
   {REINDEX reindex}
   {{ALTER TABLE} altertable}
+  {{ANALYZE} analyze}
 }] {
   foreach {s_title s_tag} $section {}
   puts "<li><a href=\"[slink $s_tag]\">$s_title</a></li>"
@@ -187,6 +188,36 @@ be readable by SQLite version 3.1.3 and earlier until the database
 is <a href="lang_vacuum.html">VACUUM</a>ed.</p>
 }
 
+Section {ANALYZE} analyze
+
+Syntax {sql-statement} {
+  ANALYZE
+}
+Syntax {sql-statement} {
+  ANALYZE <database-name>
+}
+Syntax {sql-statement} {
+  ANALYZE [<database-name> .] <table-name>
+}
+
+puts {
+<p>The ANALYZE command gathers statistics about indices and stores them
+in a special tables in the database where the query optimizer can use
+them to help make better index choices.
+If no arguments are given, all indices in all attached databases are
+analyzed.  If a database name is given as the argument, all indices
+in that one database are analyzed.  If the argument is a table name,
+then only indices associated with that one table are analyzed.</p>
+
+<p>The initial implementation stores all statistics in a single
+table named <b>sqlite_stat1</b>.  Future enhancements may create
+additional tables with the same name pattern except with the "1"
+changed to a different digit.  The <b>sqlite_stat1</b> table cannot
+be <a href="#droptable">DROP</a>ped,
+but it all the content can be <a href="#delete">DELETE</a>d which has the
+same effect.</p>
+}
+
 Section {ATTACH DATABASE} attach
 
 Syntax {sql-statement} {
@@ -409,9 +440,8 @@ puts "\"[Operator \\.]\".</p>"
 Section {CREATE INDEX} createindex
 
 Syntax {sql-statement} {
-CREATE [UNIQUE] INDEX [<database-name> .] <index-name> 
+CREATE [UNIQUE] INDEX [IF NOT EXISTS] [<database-name> .] <index-name> 
 ON <table-name> ( <column-name> [, <column-name>]* )
-[ ON CONFLICT <conflict-algorithm> ]
 } {column-name} {
 <name> [ COLLATE <collation-name>] [ ASC | DESC ]
 }
@@ -438,16 +468,6 @@ attached to a single table, nor on the number of columns in an index.</p>
 index entries are not allowed.  Any attempt to insert a duplicate entry
 will result in an error.</p>
 
-<p>The optional conflict-clause allows the specification of an alternative
-default constraint conflict resolution algorithm for this index.
-This only makes sense if the UNIQUE keyword is used since otherwise
-there are not constraints on the index.  The default algorithm is
-ABORT.  If a COPY, INSERT, or UPDATE statement specifies a particular
-conflict resolution algorithm, that algorithm is used in place of
-the default algorithm specified here.
-See the section titled
-<a href="#conflict">ON CONFLICT</a> for additional information.</p>
-
 <p>The exact text
 of each CREATE INDEX statement is stored in the <b>sqlite_master</b>
 or <b>sqlite_temp_master</b> table, depending on whether the table
@@ -456,6 +476,9 @@ all CREATE INDEX statements
 are read from the <b>sqlite_master</b> table and used to regenerate
 SQLite's internal representation of the index layout.</p>
 
+<p>If the optional IF NOT EXISTS clause is present and another index
+with the same name aleady exists, then this command becomes a no-op.</p>
+
 <p>Indexes are removed with the <a href="#dropindex">DROP INDEX</a> 
 command.</p>
 }
@@ -464,7 +487,7 @@ command.</p>
 Section {CREATE TABLE} {createtable}
 
 Syntax {sql-command} {
-CREATE [TEMP | TEMPORARY] TABLE <table-name> (
+CREATE [TEMP | TEMPORARY] TABLE [IF NOT EXISTS] <table-name> (
   <column-def> [, <column-def>]*
   [, <constraint>]*
 )
@@ -480,13 +503,13 @@ CREATE [TEMP | TEMPORARY] TABLE [<database-name>.] <table-name> AS <select-state
 NOT NULL [ <conflict-clause> ] |
 PRIMARY KEY [<sort-order>] [ <conflict-clause> ] [AUTOINCREMENT] |
 UNIQUE [ <conflict-clause> ] |
-CHECK ( <expr> ) [ <conflict-clause> ] |
+CHECK ( <expr> ) |
 DEFAULT <value> |
 COLLATE <collation-name>
 } {constraint} {
 PRIMARY KEY ( <column-list> ) [ <conflict-clause> ] |
 UNIQUE ( <column-list> ) [ <conflict-clause> ] |
-CHECK ( <expr> ) [ <conflict-clause> ]
+CHECK ( <expr> )
 } {conflict-clause} {
 ON CONFLICT <conflict-algorithm>
 }
@@ -541,15 +564,16 @@ on automatic B-Tree key generation is available
 <a href="autoinc.html">separately</a>.</p>
 
 <p>If the "TEMP" or "TEMPORARY" keyword occurs in between "CREATE"
-and "TABLE" then the table that is created is only visible to the
-process that opened the database and is automatically deleted when
-the database is closed.  Any indices created on a temporary table
+and "TABLE" then the table that is created is only visible
+within that same database connection
+and is automatically deleted when
+the database connection is closed.  Any indices created on a temporary table
 are also temporary.  Temporary tables and indices are stored in a
 separate file distinct from the main database file.</p>
 
-<p> If a <database-name> is specified, then the table is created in 
-the named database. It is an error to specify both a <database-name>
-and the TEMP keyword, unless the <database-name> is "temp". If no
+<p> If a &lt;database-name&gt; is specified, then the table is created in 
+the named database. It is an error to specify both a &lt;database-name&gt;
+and the TEMP keyword, unless the &lt;database-name&gt; is "temp". If no
 database name is specified, and the TEMP keyword is not present,
 the table is created in the main database.</p>
 
@@ -564,10 +588,8 @@ default algorithm specified in the CREATE TABLE statement.
 See the section titled
 <a href="#conflict">ON CONFLICT</a> for additional information.</p>
 
-<p>CHECK constraints are ignored in the current implementation.
-Support for CHECK constraints may be added in the future.  As of
-version 2.3.0, NOT NULL, PRIMARY KEY, and UNIQUE constraints all
-work.</p>
+<p>CHECK constraints are supported as of version 3.3.0.  Prior
+to version 3.3.0, CHECK constraints were parsed but not enforced.</p>
 
 <p>There are no arbitrary limits on the number
 of columns or on the number of constraints in a table.
@@ -592,6 +614,9 @@ The text of CREATE TEMPORARY TABLE statements are stored in the
 <b>sqlite_temp_master</b> table.
 </p>
 
+<p>If the optional IF NOT EXISTS clause is present and another table
+with the same name aleady exists, then this command becomes a no-op.</p>
+
 <p>Tables are removed using the <a href="#droptable">DROP TABLE</a> 
 statement.  </p>
 }
@@ -749,8 +774,7 @@ the statement that caused the trigger program to execute and any subsequent
 </p>
 
 <p>Triggers are removed using the <a href="#droptrigger">DROP TRIGGER</a>
-statement.  Non-temporary triggers cannot be added on a table in an 
-attached database.</p>
+statement.</p>
 }
 
 
@@ -768,7 +792,7 @@ of another SELECT in place of a table name.
 </p>
 
 <p>If the "TEMP" or "TEMPORARY" keyword occurs in between "CREATE"
-and "TABLE" then the table that is created is only visible to the
+and "VIEW" then the view that is created is only visible to the
 process that opened the database and is automatically deleted when
 the database is closed.</p>
 
@@ -782,8 +806,7 @@ the table is created in the main database.</p>
 in SQLite.  However, in many cases you can use a <a href="#createtrigger">
 TRIGGER</a> on the view to accomplish the same thing.  Views are removed 
 with the <a href="#dropview">DROP VIEW</a> 
-command.  Non-temporary views cannot be created on tables in an attached 
-database.</p>
+command.</p>
 }
 
 
@@ -825,7 +848,7 @@ others intact.</p>
 Section {DROP INDEX} dropindex
 
 Syntax {sql-command} {
-DROP INDEX [<database-name> .] <index-name>
+DROP INDEX [IF EXISTS] [<database-name> .] <index-name>
 }
 
 puts {
@@ -836,16 +859,18 @@ the disk.  The only way to recover the index is to reenter the
 appropriate CREATE INDEX command.</p>
 
 <p>The DROP INDEX statement does not reduce the size of the database 
-file.  Empty space in the database is retained for later INSERTs.  To 
+file in the default mode.
+Empty space in the database is retained for later INSERTs.  To 
 remove free space in the database, use the <a href="#vacuum">VACUUM</a> 
-command.</p>
+command.  If AUTOVACUUM mode is enabled for a database then space
+will be freed automatically by DROP INDEX.</p>
 }
 
 
 Section {DROP TABLE} droptable
 
 Syntax {sql-command} {
-DROP TABLE [<database-name>.] <table-name>
+DROP TABLE [IF EXISTS] [<database-name>.] <table-name>
 }
 
 puts {
@@ -853,13 +878,17 @@ puts {
 "#createtable">CREATE TABLE</a> statement.  The name specified is the
 table name.  It is completely removed from the database schema and the 
 disk file.  The table can not be recovered.  All indices associated 
-with the table are also deleted.  Non-temporary tables in an attached 
-database cannot be dropped.</p>
+with the table are also deleted.</p>
 
 <p>The DROP TABLE statement does not reduce the size of the database 
-file.  Empty space in the database is retained for later INSERTs.  To 
+file in the default mode.  Empty space in the database is retained for
+later INSERTs.  To 
 remove free space in the database, use the <a href="#vacuum">VACUUM</a> 
-command.</p>
+command.  If AUTOVACUUM mode is enabled for a database then space
+will be freed automatically by DROP TABLE.</p>
+
+<p>The optional IF EXISTS clause suppresses the error that would normally
+result if the table does not exist.</p>
 }
 
 
@@ -871,8 +900,7 @@ puts {
 <p>The DROP TRIGGER statement removes a trigger created by the 
 <a href="#createtrigger">CREATE TRIGGER</a> statement.  The trigger is 
 deleted from the database schema. Note that triggers are automatically 
-dropped when the associated table is dropped.  Non-temporary triggers 
-cannot be dropped on attached tables.</p>
+dropped when the associated table is dropped.</p>
 }
 
 
@@ -886,8 +914,7 @@ puts {
 <p>The DROP VIEW statement removes a view created by the <a href=
 "#createview">CREATE VIEW</a> statement.  The name specified is the 
 view name.  It is removed from the database schema, but no actual data 
-in the underlying base tables is modified.  Non-temporary views in 
-attached databases cannot be dropped.</p>
+in the underlying base tables is modified.</p>
 }
 
 
@@ -916,14 +943,14 @@ Section expression expr
 
 Syntax {expr} {
 <expr> <binary-op> <expr> |
-<expr> <like-op> <expr> [ESCAPE <expr>] |
-<expr> <glob-op> <expr> |
+<expr> [NOT] <like-op> <expr> [ESCAPE <expr>] |
 <unary-op> <expr> |
 ( <expr> ) |
 <column-name> |
 <table-name> . <column-name> |
 <database-name> . <table-name> . <column-name> |
 <literal-value> |
+<parameter> |
 <function-name> ( <expr-list> | STAR ) |
 <expr> ISNULL |
 <expr> NOTNULL |
@@ -932,11 +959,10 @@ Syntax {expr} {
 <expr> [NOT] IN ( <select-statement> ) |
 <expr> [NOT] IN [<database-name> .] <table-name> |
 [EXISTS] ( <select-statement> ) |
-CASE [<expr>] LP WHEN <expr> THEN <expr> RPPLUS [ELSE <expr>] END
+CASE [<expr>] LP WHEN <expr> THEN <expr> RPPLUS [ELSE <expr>] END |
+CAST ( <expr> AS <type> )
 } {like-op} {
-LIKE | NOT LIKE
-} {glob-op} {
-GLOB | NOT GLOB
+LIKE | GLOB | REGEXP
 }
 
 puts {
@@ -965,12 +991,7 @@ OR</font>
 <font color="#2c2cf0"><big>-    +    !    ~</big></font>
 </pre></blockquote>
 
-<p>Any SQLite value can be used as part of an expression.  
-For arithmetic operations, integers are treated as integers.
-Strings are first converted to real numbers using <b>atof()</b>.
-For comparison operators, numbers compare as numbers and strings
-compare using the <b>strcmp()</b> function.
-Note that there are two variations of the equals and not equals
+<p>Note that there are two variations of the equals and not equals
 operators.  Equals can be either}
 puts "[Operator =] or [Operator ==].
 The non-equals operator can be either
@@ -978,9 +999,77 @@ The non-equals operator can be either
 The [Operator ||] operator is \"concatenate\" - it joins together
 the two strings of its operands.
 The operator [Operator %] outputs the remainder of its left 
-operand modulo its right operand.</p>"
+operand modulo its right operand.</p>
+
+<p>The result of any binary operator is a numeric value, except
+for the [Operator ||] concatenation operator which gives a string
+result.</p>"
+
 puts {
 
+<a name="literal_value"></a>
+<p>
+A literal value is an integer number or a floating point number.
+Scientific notation is supported.  The "." character is always used
+as the decimal point even if the locale setting specifies "," for
+this role - the use of "," for the decimal point would result in
+syntactic ambiguity.  A string constant is formed by enclosing the
+string in single quotes (').  A single quote within the string can
+be encoded by putting two single quotes in a row - as in Pascal.
+C-style escapes using the backslash character are not supported because
+they are not standard SQL.
+BLOB literals are string literals containing hexadecimal data and
+preceded by a single "x" or "X" character.  For example:</p>
+
+<blockquote><pre>
+X'53514697465'
+</pre></blockquote>
+
+<p>
+A literal value can also be the token "NULL".
+</p>
+
+<p>
+A parameter specifies a placeholder in the expression for a literal
+value that is filled in at runtime using the
+<a href="capi3ref.html#sqlite3_bind_int">sqlite3_bind</a> API.
+Parameters can take several forms:
+</p
+
+<blockquote>
+<table>
+<tr>
+<td align="right" valign="top"><b>?</b><i>NNN</i></td><td width="20"></td>
+<td>A question mark followed by a number <i>NNN</i> holds a spot for the
+NNN-th parameter.  NNN must be between 1 and 999.</td>
+</tr>
+<tr>
+<td align="right" valign="top"><b>?</b></td><td width="20"></td>
+<td>A question mark that is not followed by a number holds a spot for
+the next unused parameter.</td>
+</tr>
+<tr>
+<td align="right" valign="top"><b>:</b><i>AAAA</i></td><td width="20"></td>
+<td>A colon followed by an identifier name holds a spot for a named
+parameter with the name AAAA.  Named parameters are also numbered.
+The number assigned is the next unused number.  To avoid confusion,
+it is best to avoid mixing named and numbered parameters.</td>
+</tr>
+<tr>
+<td align="right" valign="top"><b>$</b><i>AAAA</i></td><td width="20"></td>
+<td>A dollar-sign followed by an identifier name also holds a spot for a named
+parameter with the name AAAA.  The identifier name in this case can include
+one or more occurances of "::" and a suffix enclosed in "(...)" containing
+any text at all.  This syntax is the form of a variable name in the Tcl
+programming language.</td>
+</tr>
+</table>
+</blockquote>
+
+<p>Parameters that are not assigned values using
+<a href="capi3ref.html#sqlite3_bind_int">sqlite3_bind</a> are treated
+as NULL.</p>
+
 <a name="like"></a>
 <p>The LIKE operator does a pattern matching comparison. The operand
 to the right contains the pattern, the left hand operand contains the
@@ -1014,9 +1103,14 @@ characters on one side against lower case characters on the other.
 characters.  Hence the LIKE operator is case sensitive for 8-bit
 iso8859 characters or UTF-8 characters.  For example, the expression
 <b>'a'&nbsp;LIKE&nbsp;'A'</b> is TRUE but
-<b>'&aelig;'&nbsp;LIKE&nbsp;'&AElig;'</b> is FALSE.).  The infix LIKE
-operator is identical the user function <a href="#likeFunc">
-like(<i>X</i>,<i>Y</i>)</a>.
+<b>'&aelig;'&nbsp;LIKE&nbsp;'&AElig;'</b> is FALSE.).</p>
+
+<p>The infix LIKE
+operator is implemented by calling the user function <a href="#likeFunc">
+like(<i>X</i>,<i>Y</i>)</a>.  If an ESCAPE clause is present, it adds
+a third parameter to the function call. If the functionality of LIKE can be
+overridden by defining an alternative implementation of the
+like() SQL function.</p>
 </p>
 
 <a name="glob"></a>
@@ -1024,8 +1118,17 @@ like(<i>X</i>,<i>Y</i>)</a>.
 file globbing syntax for its wildcards.  Also, GLOB is case
 sensitive, unlike LIKE.  Both GLOB and LIKE may be preceded by
 the NOT keyword to invert the sense of the test.  The infix GLOB 
-operator is identical the user function <a href="#globFunc">
-glob(<i>X</i>,<i>Y</i>)</a>.</p>
+operator is implemented by calling the user function <a href="#globFunc">
+glob(<i>X</i>,<i>Y</i>)</a> and can be modified by overriding
+that function.</p>
+
+<a name="regexp"></a>
+<p>The REGEXP operator is a special syntax for the regexp()
+user function.  No regexp() user function is defined by default
+and so use of the REGEXP operator will normally result in an
+error message.  If a user-defined function named "regexp"
+is defined at run-time, that function will be called in order
+to implement the REGEXP operator.</p>
 
 <p>A column name can be any of the names defined in the CREATE TABLE
 statement or one of the following special identifiers: "<b>ROWID</b>",
@@ -1067,6 +1170,11 @@ SELECT becomes the value used in the expression.  If the SELECT yields
 more than one result row, all rows after the first are ignored.  If
 the SELECT yields no rows, then the value of the SELECT is NULL.</p>
 
+<p>A CAST expression changes the datatype of the <expr> into the
+type specified by &lt;type&gt;. 
+&lt;type&gt; can be any non-empty type name that is valid
+for the type in a column definition of a CREATE TABLE statement.</p>
+
 <p>Both simple and aggregate functions are supported.  A simple
 function can be used in any expression.  Simple functions return
 a result immediately based on their inputs.  Aggregate functions
@@ -1099,7 +1207,7 @@ all arguments are NULL then NULL is returned.  There must be at least
 <a href="capi3ref.html#sqlite3_create_function">sqlite3_create_function()</a> 
 interface can
 be used to override this function and thereby change the operation
-of the <a href="#glob">GLOB</a> operator.</td>
+of the <a href="#globFunc">GLOB</a> operator.</td>
 </tr>
 
 <tr>
@@ -1240,15 +1348,28 @@ UTF-8 strings.</td>
 </table>
 
 <p>
-The following aggregate functions are available by default.  Additional
+The aggregate functions shown below are available by default.  Additional
 aggregate functions written in C may be added using the 
 <a href="capi3ref.html#sqlite3_create_function">sqlite3_create_function()</a>
 API.</p>
 
+<p>
+In any aggregate function that takes a single argument, that argument
+can be preceeded by the keyword DISTINCT.  In such cases, duplicate
+elements are filtered before being passed into the aggregate function.
+For example, the function "count(distinct X)" will return the number
+of distinct values of column X instead of the total number of non-null
+values in column X.
+</p>
+
 <table border=0 cellpadding=10>
 <tr>
 <td valign="top" align="right" width=120>avg(<i>X</i>)</td>
-<td valign="top">Return the average value of all <i>X</i> within a group.</td>
+<td valign="top">Return the average value of all non-NULL <i>X</i> within a
+group.  String and BLOB values that do not look like numbers are
+interpreted as 0.
+The result of avg() is always a floating point value even if all
+inputs are integers. </p></td>
 </tr>
 
 <tr>
@@ -1272,8 +1393,24 @@ if all values in the group are NULL.</td>
 </tr>
 
 <tr>
-<td valign="top" align="right">sum(<i>X</i>)</td>
-<td valign="top">Return the numeric sum of all values in the group.</td>
+<td valign="top" align="right">sum(<i>X</i>)<br>total(<i>X</i>)</td>
+<td valign="top">Return the numeric sum of all non-NULL values in the group.
+   If there are no non-NULL input rows then sum() returns
+   NULL but total() returns 0.0.
+   NULL is not normally a helpful result for the sum of no rows
+   but the SQL standard requires it and most other
+   SQL database engines implement sum() that way so SQLite does it in the
+   same way in order to be compatible.   The non-standard total() function
+   is provided as a convenient way to work around this design problem
+   in the SQL language.</p>
+
+   <p>The result of total() is always a floating point value.
+   The result of sum() is an integer value if all non-NULL inputs are integers
+   and the sum is exact.  If any input to sum() is neither an integer or
+   a NULL or if the
+   an integer overflow occurs at any point during the computation,
+   then sum() returns a floating point value
+   which might be an approximation to the true sum.</td>
 </tr>
 </table>
 }
@@ -1331,9 +1468,11 @@ It is given its own section in this document because it is not
 part of standard SQL and therefore might not be familiar.</p>
 
 <p>The syntax for the ON CONFLICT clause is as shown above for
-the CREATE TABLE and CREATE INDEX commands.  For the COPY, INSERT, and
+the CREATE TABLE command.  For the INSERT and
 UPDATE commands, the keywords "ON CONFLICT" are replaced by "OR", to make
-the syntax seem more natural.  But the meaning of the clause is the same
+the syntax seem more natural.  For example, instead of
+"INSERT ON CONFLICT IGNORE" we have "INSERT OR IGNORE".
+The keywords change but the meaning of the clause is the same
 either way.</p>
 
 <p>The ON CONFLICT clause specifies an algorithm used to resolve
@@ -1386,12 +1525,10 @@ satisfy a constraint, it does not invoke delete triggers on those
 rows.  But that may change in a future release.</p>
 </dl>
 
-<p>The algorithm specified in the OR clause of a COPY, INSERT, or UPDATE
-overrides any algorithm specified in a CREATE TABLE or CREATE INDEX.
+<p>The algorithm specified in the OR clause of a INSERT or UPDATE
+overrides any algorithm specified in a CREATE TABLE.
 If no algorithm is specified anywhere, the ABORT algorithm is used.</p>
 }
-# <p>For additional information, see 
-# <a href="conflict.html">conflict.html</a>.</p>
 
 Section REINDEX reindex
 
@@ -1404,8 +1541,7 @@ Syntax {sql-statement} {
 
 puts {
 <p>The REINDEX command is used to delete and recreate indices from scratch.
-This is primarily useful when the definition of a collation sequence has 
-changed.
+This is useful when the definition of a collation sequence has changed.
 </p>
 
 <p>In the first form, all indices in all attached databases that use the
@@ -1542,7 +1678,7 @@ a single big table.  The difference is that in UNION all result rows
 are distinct where in UNION ALL there may be duplicates.
 The INTERSECT operator takes the intersection of the results of the
 left and right SELECTs.  EXCEPT takes the result of left SELECT after
-removing the results of the right SELECT.  When three are more SELECTs
+removing the results of the right SELECT.  When three or more SELECTs
 are connected into a compound, they group from left to right.</p>
 }
 
